Package: Lheuristic
Title: Detection of scatterplots with L-shaped pattern
Version: 1.2.0
Authors@R: c(
    person("Sanchez Pla", "Alex", email ="asanchez@ub.edu", role = c("aut", "cre"),
  comment = c(ORCID = "0000-0002-8673-7737")),
    person("Miro Cau", "Berta", email = "berta.miro@vhir.org", role = "aut", ,
  comment = c(ORCID = "0000-0001-6049-8697")))
Description: The Lheuristic package identifies scatterpots that
    follow and L-shaped, negative distribution. It can be used
    to identify genes regulated by methylation by integration of
    an expression and a methylation array. The package uses two
    different methods to detect expression and methyaltion L-
    shapped scatterplots. The parameters can be changed to
    detect other scatterplot patterns.
License: MIT + file LICENSE
Depends: R (>= 4.4.0)
biocViews: DNAMethylation, StatisticalMethod, MethylationArray
Imports: Hmisc, stats, energy, grDevices, graphics, utils,
        MultiAssayExperiment, ggplot2, ggpubr
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
URL: https://github.com/ASPresearch/Lheuristic
BugReports: https://github.com/ASPresearch/Lheuristic/issues
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.3.2
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/Lheuristic
git_branch: RELEASE_3_22
git_last_commit: ef97be8
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:44:00 UTC; biocbuild
Author: Sanchez Pla Alex [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8673-7737>),
  Miro Cau Berta [aut] (ORCID: <https://orcid.org/0000-0001-6049-8697>)
Maintainer: Sanchez Pla Alex <asanchez@ub.edu>

# Generated by roxygen2: do not edit by hand

export(binScore)
export(calcFreqs)
export(checkPairing)
export(correlationSelection)
export(lhCreateMAE)
export(matCorrs)
export(messageTitle)
export(numScore)
export(plotGeneByName)
export(plotGeneSel)
export(plotGenesMat)
export(scoreGenesMat)
export(toReqMat)
import(Hmisc)
import(ggplot2)
import(ggpubr)
import(stats)
importFrom(MultiAssayExperiment,ExperimentList)
importFrom(MultiAssayExperiment,MultiAssayExperiment)
importFrom(MultiAssayExperiment,assay)
importFrom(MultiAssayExperiment,experiments)
importFrom(energy,dcor)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,abline)

The Lheuristic package identifies scatterpots that
    follow and L-shaped, negative distribution. It can be used
    to identify genes regulated by methylation by integration of
    an expression and a methylation array. The package uses two
    different methods to detect expression and methyaltion L-
    shapped scatterplots. The parameters can be changed to
    detect other scatterplot patterns.

#' toReqMat
#'
#' \code{toReqMat} can be used to turn a matrix of required percentages into
#' a matrix of required counts to facilitate its scoring.
#'
#' @param numPoints Number of points in a scatterplot. Used to turn the required
#' percentages into required counts.
#' @param aReqPercentMat Matrix of required percentages.
#'
#' @return a counts matrix
#'
#' @export toReqMat
#'
#' @examples
#' reqPercentages <- matrix(
#'     c(
#'         3, 20, 5,
#'         5, 40, 20,
#'         4, 1, 2
#'     ),
#'     nrow = 3, byrow = TRUE
#' )
#' numberOfPoints <- 100
#' reqMat <- toReqMat(
#'     numPoints = numberOfPoints,
#'     aReqPercentMat = reqPercentages
#' )
#'
toReqMat <- function(numPoints, aReqPercentMat) {
    return(round(aReqPercentMat * numPoints / 100, 0))
}

#' plotGeneByName
#'
#' \code{plotGeneByName} plots points on a scatterplot with a 3x3 grid
#' superimposed. The name of a the gene is provided jointly with the
#' MultiAssayExperiment object and used to select the row to be plotted.
#'
#' @param geneName The name of the gene to be plotted.
#' @param  mae A MultiAssayExperiment object containing the methylation
#' and expression data for the specified gene.
#' @param filename If provided, the name of the file to save the results
#' as a PDF; defaults to NULL.
#' @param text4Title A string used as the main title for the plot. Defaults to 
#' `geneName` if set to NULL.
#' @param plotGrid A boolean parameter indicating whether to pass
#' the grid option to the `plotGeneSel` function.
#' @param figs A two-component vector defining the 2-dimensional structure of
#' the plots to be generated.
#'
#' @return a pdf with scatterplots of selected by gene name
#'
#' @keywords plot gene name
#' @importFrom grDevices pdf
#' @importFrom grDevices dev.off
#' @importFrom MultiAssayExperiment assay
#' @import ggplot2
#' @export plotGeneByName
#'
#' @examples
#' # Plot gene by name based on example data
#' # Methylation data
#' methylData <- matrix(runif(50), nrow = 10)
#' colnames(methylData) <- paste0("samp", 1:ncol(methylData))
#' rownames(methylData) <- paste0("gene", 1:nrow(methylData))
#' # Expression data
#' expresData <- matrix(rnorm(50), nrow = 10)
#' colnames(expresData) <- paste0("samp", 1:ncol(methylData))
#' rownames(expresData) <- paste0("gene", 1:nrow(methylData))
#' # ColData
#' colDat <- data.frame(
#'     sampleID = colnames(methylData),
#'     name = letters[1:ncol(methylData)]
#' )
#'
#' rownames(colDat) <- colDat$sampleID
#' mae <- MultiAssayExperiment::MultiAssayExperiment(
#'     experiments = list(
#'         methylation = methylData,
#'         expression = expresData
#'     ),
#'     colData = colDat
#' )
#' plotGeneByName(gene = "gene7", mae = mae)
#'
plotGeneByName <- function(geneName, mae, filename = NULL,
    text4Title = NULL,
    plotGrid = TRUE, figs = c(2, 2)) {
    # Open PDF if filename is provided
    if (!is.null(filename)) {
    grDevices::pdf(filename)
    }

    # Set plot title   
    plotTitle <- if (!is.null(text4Title)) paste(geneName, text4Title, 
    sep = ", ") else geneName

    # Check if the gene name exists in the expression data 
    #and get the gene position
    if (geneName %in% rownames(assay(mae, "expression"))) {
    genePos <- which(rownames(assay(mae, "expression")) == geneName)
    } else {
    stop("Gene not found in expression data.")
    }

    # Plot using plotGeneSel with ggplot2
    plotGeneSel(
    mae = mae, genePos = genePos, titleText = plotTitle,
    x1 = 1/3, x2 = 2/3, plotGrid = plotGrid
    )

    # Close PDF if filename is provided
    if (!is.null(filename)) {
    grDevices::dev.off()
    }
}

#' binScore
#'
#' \code{binScore} can be used to score scatterplots by directly comparing
#' the sample counts with a matrix of minimal or maximal percentages/counts
#' to be found in each cell. It implements the three bands rule implicitly
#' by setting threshold values.
#'
#' @param aGrid A matrix of counts as computed by `calcFreqs` function.
#' @param aReq A matrix of minimum or maximum counts to be found in each cell
#' if L-shape is TRUE.
#'
#' @return a score value for each scatterplot
#'
#' @keywords binary scoring
#' @export binScore
#' @examples
#' # Generate some example data
#' aGrid <- matrix(c(20, 3, 0, 10, 2, 2, 20, 10, 20),
#'     nrow = 3, ncol = 3, byrow = TRUE
#' )
#' aReq <- matrix(c(15, 5, 0, 0, 5, 5, 10, 10, 15),
#'     nrow = 3, ncol = 3, byrow = TRUE
#' )
#'
#' # Calculate the bin score
#' binScore(aGrid, aReq)
#'
binScore <- function(aGrid, aReq) {
    comp <- (aGrid[1, 1] >= aReq[1, 1]) &&
            (aGrid[1, 2] <= aReq[1, 2]) && (aGrid[
        1,
        3
    ] <= aReq[1, 3]) && (aGrid[2, 1] >= aReq[2, 1]) &&
                        (aGrid[2, 2] <= aReq[
        2,
        2
    ]) && (aGrid[2, 3] <= aReq[2, 3]) && 
            (aGrid[3, 1] >= aReq[3, 1]) && (aGrid[
        3,
        2
    ] >= aReq[3, 2]) && (aGrid[3, 3] >= aReq[3, 3])
    return(comp)
}

# Test that binScore returns the expected output for a given input
library(testthat)
test_that("binScore returns the expected output for a given input", {
    # Generate example data
    aGrid <- matrix(c(20, 3, 0, 10, 2, 2, 20, 10, 20), nrow = 3, ncol = 3, byrow = TRUE)
    aReq <- matrix(c(15, 5, 0, 0, 5, 5, 10, 10, 15), nrow = 3, ncol = 3, byrow = TRUE)

    # Test binScore with the example data
    expected_output <- TRUE
    output <- binScore(aGrid, aReq)
    expect_equal(output, expected_output, info = "binScore should return TRUE for the example input data")
})

#' calcFreqs
#'
#' \code{calcFreqs} Given a MultiAssayExperiment with methylation and
#' expression data, the function overlays a grid on a YMet ~ Xmet scatterplot
#' and returns a 3x3 matrix with point counts per grid cell based on  the
#' vertical and horizontal lines.
#'
#' @param mae MultiAssayExperiment object containing methylation and expression
#' matrices.
#' @param geneNum row of expression/methylation matrix for which the frequencies
#' will be computed.
#' @param x1,x2 Coordinates of vertical points in the X axis. Because it is
#' expected to contain.
#' methylation values that vary between 0 and 1, default values are 1/3 and 2/3.
#' @param y1,y2 Coordinates of vertical points on the Y-axis. If set to NULL,
#' they will be automatically assigned as the percentiles of `y` defined 
#' by `percY1` and `percY2`.
#' @param percY1,percY2 Default values for `y1` and `y2` when set to `NULL`
#'
#' @return a matrix with calculated frequencies
#'
#' @keywords calculation frequencies
#' @export calcFreqs
#'
#' @examples
#'
#' # Methylation data
#' methylData <- matrix(runif(50), nrow = 10)
#' colnames(methylData) <- paste0(
#'     "samp",
#'     1:ncol(methylData)
#' )
#' rownames(methylData) <- paste0(
#'     "gene",
#'     1:nrow(methylData)
#' )
#' # Expression data
#' expresData <- matrix(rnorm(50), nrow = 10)
#' colnames(expresData) <- paste0(
#'     "samp",
#'     1:ncol(methylData)
#' )
#' rownames(expresData) <- paste0(
#'     "gene",
#'     1:nrow(methylData)
#' )
#' # ColData
#' colDat <- data.frame(
#'     sampleID = colnames(methylData),
#'     name = letters[1:ncol(methylData)]
#' )
#'
#' rownames(colDat) <- colDat$sampleID
#' mae <- MultiAssayExperiment::MultiAssayExperiment(
#'     experiments = list(
#'         methylation = methylData,
#'         expression = expresData
#'     ),
#'     colData = colDat
#' )
#' geneRow <- 1
#' x1 <- 1 / 3
#' x2 <- 2 / 3
#' y1 <- NULL
#' y2 <- NULL
#' percY1 <- 1 / 3
#' percY2 <- 2 / 3
#'
#' calcFreqs(
#'     mae = mae, geneNum = geneRow,
#'     x1, x2, y1, y2, percY1, percY2
#' )
calcFreqs <- function(mae, geneNum, x1, x2, y1 = NULL, y2 = NULL,
    percY1 = 1 / 3, percY2 = 2 / 3) {
    # Extract standardized assay names from mae
    assayNames <- names(MultiAssayExperiment::experiments(mae))
    xName <- assayNames[1]
    yName <- assayNames[2]
    # Extract methylation and expression data
    mets <- MultiAssayExperiment::assay(mae, xName)
    expres <- MultiAssayExperiment::assay(mae, yName)
    xMet <- mets[geneNum, ]
    yExp <- expres[geneNum, ]
    freqsMat <- matrix(0, nrow = 3, ncol = 3)
    xVals <- c(x1, x2)
    minExp <- min(yExp)
    maxExp <- max(yExp)
    delta <- maxExp - minExp
    if (is.null(y1)) {
        y1 <- minExp + percY1 * delta
    }
    if (is.null(y2)) {
        y2 <- minExp + percY2 * delta
    }
    yVals <- c(y1, y2)
    condX <- c("(xMet<=x1)", "((xMet>x1) &
                (xMet<=x2))", "(xMet>x2)")
    condY <- c("(yExp>y2)", "((yExp<=y2) &
                (yExp>y1))", "(yExp<=y1)")
    for (i in seq_len(3)) {
        for (j in seq_len(3)) {
            condij <- paste(condX[j], condY[i], sep = "&")
            freqsMat[i, j] <- sum(eval(parse(text = condij)))
        }
    }
    return(freqsMat)

}

library(testthat)
library(MultiAssayExperiment)

# Create example data
set.seed(123)
methylData <- matrix(runif(50), nrow = 10)
colnames(methylData) <- paste0("samp", 1:ncol(methylData))
rownames(methylData) <- paste0("gene", 1:nrow(methylData))

expresData <- matrix(rnorm(50), nrow = 10)
colnames(expresData) <- paste0("samp", 1:ncol(methylData))
rownames(expresData) <- paste0("gene", 1:nrow(methylData))

colDat <- data.frame(sampleID = colnames(methylData), name = letters[1:ncol(methylData)])
rownames(colDat) <- colDat$sampleID

mae <- lhCreateMAE(methylData, expresData, colData = colDat)

test_that("calcFreqs returns a 3x3 matrix with expected properties", {
  # Set parameters
    geneRow <- 1
    x1 <- 1 / 3
    x2 <- 2 / 3
    y1 <- NULL
    y2 <- NULL
    percY1 <- 1 / 3
    percY2 <- 2 / 3
    
    # Run function
    freqs <- calcFreqs(mae, geneRow, x1, x2, y1, y2, percY1, percY2)
  
  # Tests
  expect_type(freqs, "double")  # Expect numeric matrix
  expect_equal(dim(freqs), c(3, 3))  # Expect 3x3 matrix
  expect_true(all(freqs >= 0))  # Expect non-negative values
  expect_equal(sum(freqs), ncol(methylData))  # Total count should match the number of samples
})

test_that("calcFreqs handles out-of-bounds geneNum", {

  numGenes <- nrow(MultiAssayExperiment::assay(mae, 1))  # Get number of genes
  
  # Test valid geneNum (should not throw errors)
  expect_silent(calcFreqs(mae, 1, 1/3, 2/3))  # First row
  expect_silent(calcFreqs(mae, numGenes, 1/3, 2/3))  # Last row
  
  # Test out-of-bounds geneNum
  warnings <- testthat::capture_warnings(calcFreqs(mae, 0, 1/3, 2/3))
  
  # Check that the specific warnings appear
  expect_true(any(grepl("no non-missing arguments to min", warnings)))
  expect_true(any(grepl("no non-missing arguments to max", warnings)))

  expect_error(calcFreqs(mae, numGenes + 1, 1/3, 2/3), regexp = "subscript out of bounds")
  
  # Alternative: Catch any generic warnings without expecting a specific message
  # expect_warning(calcFreqs(mae, 0, 1/3, 2/3))
  # expect_error(calcFreqs(mae, numGenes + 1, 1/3, 2/3))
})

#' checkPairing
#'
#' \code{checkPairing} function to check if two matrices have the same 
#' dimensions row and column names.
#'
#' @param X matrix
#' @param Y matrix
#'
#' @return a logical value indicating if the two matrices have the 
#' same row and column names.
#'
#' @keywords matrix
#'
#' @export checkPairing
#'
#' @examples
#' (X <- round(matrix(rnorm(30) * 10, ncol = 6), 1)) + 1:10
#' (Y <- round(X + matrix(rnorm(30) * 10, ncol = 6), 1)) - 10:1
#' (rownames(X) <- rownames(Y) <- letters[1:nrow(X)])
#' (m1 <- checkPairing(X, Y))
#'
checkPairing <- function(X, Y) {
    # Check if row names and column names match
    row_check <- sum(rownames(X) != rownames(Y)) == 0
    col_check <- sum(colnames(X) != colnames(Y)) == 0

    # Return TRUE if both rows and columns match, FALSE otherwise
    return(row_check && col_check)
}

#' A row-wise correlation function calculator
#'
#' \code{matCorrs} Given a MultiAssayExperiment object, the function
#' computes Pearson and Spearman correlation coefficients
#' and their significance p-values for every pair of row vectors.
#' @param mae A MultiAssayExperiment object containing the methylation
#' and expression data.
#'
#' @return matrix with a correlation value for each gene
#'
#' @keywords correlation
#'
#' @import Hmisc
#' @export matCorrs
#' @examples
#' # Methylation data
#' methylData <- matrix(runif(50), nrow = 10)
#' colnames(methylData) <- paste0("samp", 1:ncol(methylData))
#' rownames(methylData) <- paste0("gene", 1:nrow(methylData))
#' # Expression data
#' expresData <- matrix(rnorm(50), nrow = 10)
#' colnames(expresData) <- paste0("samp", 1:ncol(methylData))
#' rownames(expresData) <- paste0("gene", 1:nrow(methylData))
#' # ColData
#' colDat <- data.frame(
#'     sampleID = colnames(methylData),
#'     name = letters[1:ncol(methylData)]
#' )
#'
#' rownames(colDat) <- colDat$sampleID
#' mae <- MultiAssayExperiment::MultiAssayExperiment(
#'     experiments = list(
#'         methylation = methylData,
#'         expression = expresData
#'     ),
#'     colData = colDat
#' )
#' matCorrs(mae)
#'
matCorrs <- function(mae) {
    corrsList <- matrix(NA, nrow = nrow(assay(mae, "methylation")), ncol = 4)
    colnames(corrsList) <- c("r (Sp)", "r (Pear)", "p (Sp)", "p (Pear)")
    rownames(corrsList) <- rownames(assay(mae, "methylation"))

    for (i in seq_len(nrow(assay(mae, "methylation")))) {
        xVec <- as.numeric(assay(mae, "methylation")[i, ])
        yVec <- as.numeric(assay(mae, "expression")[i, ])
        corrs <- vecCorrs(xVec, yVec)
        corrsList[i, ] <- corrs
    }

    return(corrsList)
}

#' numScore
#'
#' \code{numScore} A function to score scatterplot using a weight matrix.
#' The scoring does not incorporate logical conditions such as 'if xij < C ...'
#' @param aGrid A matrix of counts computed by the `calcFreqs` function.
#' @param LShaped A boolean indicating whether the scatterplot can be
#' classified as "L-shaped".
#' @param aWeightMifL A matrix of weights applied to score counts if
#' the scatterplot is classified as "L".
#' @param aWeightMifNonL A matrix of weights applied to score counts if
#' the scatterplot is classified as "non-L".
#'
#' @return a numeric score for each scatterplot
#'
#' @keywords scatterplot weights
#'
#' @export numScore
#' @examples
#' # Methylation data
#' methylData <- matrix(runif(50), nrow = 10)
#' colnames(methylData) <- paste0("samp", 1:ncol(methylData))
#' rownames(methylData) <- paste0("gene", 1:nrow(methylData))
#' # Expression data
#' expresData <- matrix(rnorm(50), nrow = 10)
#' colnames(expresData) <- paste0("samp", 1:ncol(methylData))
#' rownames(expresData) <- paste0("gene", 1:nrow(methylData))
#' # ColData
#' colDat <- data.frame(
#'     sampleID = colnames(methylData),
#'     name = letters[1:ncol(methylData)]
#' )
#'
#' rownames(colDat) <- colDat$sampleID
#' mae <- MultiAssayExperiment::MultiAssayExperiment(
#'     experiments = list(
#'         methylation = methylData,
#'         expression = expresData
#'     ),
#'     colData = colDat
#' )
#' trueFreq <- calcFreqs(mae, x1 = 1 / 3, x2 = 2 / 3)
#' LShaped <- FALSE
#' weightsIfL <- matrix(c(2, -1, -99, 1, 0, -1, 1, 1, 2),
#'     nrow = 3, byrow = TRUE
#' )
#' weightsIfNonL <- matrix(c(0, -1, -99, 0, 0, -1, 0, 0, 0),
#'     nrow = 3, byrow = TRUE
#' )
#' numScore(
#'     trueFreq, LShaped,
#'     weightsIfL, weightsIfNonL
#' )
#'
numScore <- function(aGrid, LShaped, aWeightMifL, aWeightMifNonL) {
    if (!LShaped) {
        scoresM <- aGrid * aWeightMifNonL
    } else {
        scoresM <- aGrid * aWeightMifL
    }
    return(sum(scoresM))
}

#' messageTitle
#'
#' \code{messageTitle} A  wrapper function for sending messages to console.
#' @param aMessage text of the message.
#' @param underChar character to use for underlining the message.
#'
#' @return a message to console
#' @export messageTitle
#'
#' @examples
#' messageTitle("Hello world", "$")
#'
messageTitle <- function(aMessage, underChar = "-") {
    message(aMessage, "\n")
    message(paste(rep(underChar, nchar(aMessage)), collapse = ""), "\n")
}

#' scoreGenesMat
#'
#' \code{scoreGenesMat} scores scatterplots using a binary and a numeric
#' schemes row-wise.
#' @param mae MultiAssayExperiment object containing methylation and expression 
#' matrices.
#' @param aReqPercentsMat A matrix specifying the minimum and maximum percentage
#' counts required in each cell.
#' @param aWeightMifL A matrix of weights applied to score counts when the
#' scatterplot is classified as "L".
#' @param aWeightMifNonL A matrix of weights applied to score counts when the
#' scatterplot is classified as "non-L".
#' @param x1,x2 Coordinates of vertical points on the X-axis. 
#' Expected to contain methylation values ranging between 0 and 1,
#' with default values set to 1/3 and 2/3.
#' @param y1,y2 Coordinates of vertical points on the Y-axis. If set to NULL,
#' they default to the percentiles of yVec as defined by `percY1` and `percY2`.
#' @param percY1,percY2 Values used to act as default for `y1`and `y2` when
#' these are set to `NULL`.
#' @export scoreGenesMat
#' @return A data frame with two columns: 'logicSc' (logical score indicating if
#' the gene is 'active') and 'numericSc' (a numerical score).
#'
#' @examples
#' # Score genes based on example data
#'
#' # Methylation data
#' methylData <- matrix(runif(50), nrow = 10)
#' colnames(methylData) <- paste0("samp", 1:ncol(methylData))
#' rownames(methylData) <- paste0("gene", 1:nrow(methylData))
#' # Expression data
#' expresData <- matrix(rnorm(50), nrow = 10)
#' colnames(expresData) <- paste0("samp", 1:ncol(methylData))
#' rownames(expresData) <- paste0("gene", 1:nrow(methylData))
#' # ColData
#' colDat <- data.frame(
#'     sampleID = colnames(methylData),
#'     name = letters[1:ncol(methylData)]
#' )
#'
#' rownames(colDat) <- colDat$sampleID
#' mae <- MultiAssayExperiment::MultiAssayExperiment(
#'     experiments = list(
#'         methylation = methylData,
#'         expression = expresData
#'     ),
#'     colData = colDat
#' )
#' sampleSize <- ncol(MultiAssayExperiment::experiments(mae)[[1]])
#' reqPercentages <- matrix(c(3, 20, 5, 5, 40, 20, 4, 1, 2),
#'     nrow = 3, byrow = TRUE
#' )
#' (theWeightMifL <- matrix(c(2, -2, -sampleSize / 5, 1, 0, -2, 1, 1, 2),
#'     nrow = 3, byrow = TRUE
#' ))
#' (theWeightMifNonL <- matrix(c(0, -2, -sampleSize / 5, 0, 0, -2, 0, 0, 0),
#'     nrow = 3, byrow = TRUE
#' ))
#' scoreGenesMat(mae,
#'     x1 = 1 / 3, x2 = 2 / 3,
#'     y1 = NULL, y2 = NULL, percY1 = 1 / 3, percY2 = 2 / 3,
#'     aReqPercentsMat = reqPercentages,
#'     aWeightMifL = theWeightMifL,
#'     aWeightMifNonL = theWeightMifNonL
#' )
scoreGenesMat <- function(mae, x1 = 1 / 3, x2 = 2 / 3, y1 = NULL, y2 = NULL,
    percY1 = 1 / 3,
    percY2 = 2 / 3, aReqPercentsMat, aWeightMifL = 0.5, aWeightMifNonL = 0.25) {
    if (sum(aReqPercentsMat) != 100) {
        stopifnot(`Percentages must add up to 100` 
            = sum(aReqPercentsMat) == 100)
    }
    # Extract standardized assay names from mae
    assayNames <- names(MultiAssayExperiment::experiments(mae))
    xName <- assayNames[1]
    yName <- assayNames[2]
    # Extract methylation and expression data
    mets <- MultiAssayExperiment::assay(mae, xName)
    expres <- MultiAssayExperiment::assay(mae, yName)
    N <- ncol(mets)
    Ngenes <- nrow(mets)
    scores <- data.frame(logicSc = rep(FALSE, Ngenes),
        numericSc = rep(0, Ngenes))
    rownames(scores) <- rownames(mets)
    minmaxCounts <- toReqMat(N, aReqPercentMat = aReqPercentsMat)
    indexes <- seq(from = 1, to = Ngenes, by = 1)
    for (gene in indexes) {
        geneNum <- rownames(expres)[gene]
        xMet <- mets[geneNum, ]
        yExp <- expres[geneNum, ]
        geneGrid <- calcFreqs(mae, geneNum, x1, x2,
            y1 = NULL, y2 = NULL, percY1 = 1 / 3,
            percY2 = 2 / 3
        )
        binSc <- binScore(geneGrid, minmaxCounts)
        scores[gene, "logicSc"] <- binSc
        numSc <- numScore(geneGrid, LShaped = binSc, aWeightMifL = aWeightMifL,
            aWeightMifNonL = aWeightMifNonL)
        scores[gene, "numericSc"] <- numSc
    }
    return(scores)
}

library(testthat)
library(MultiAssayExperiment)

test_that("scoreGenesMat generates valid scores", {
  # Generate test data
  set.seed(42)
  methylData <- matrix(runif(50), nrow = 10)
  colnames(methylData) <- paste0("samp", 1:ncol(methylData))
  rownames(methylData) <- paste0("gene", 1:nrow(methylData))
  
  expresData <- matrix(rnorm(50), nrow = 10)
  colnames(expresData) <- paste0("samp", 1:ncol(methylData))
  rownames(expresData) <- paste0("gene", 1:nrow(methylData))
  
  colDat <- data.frame(sampleID = colnames(methylData), name = letters[1:ncol(methylData)])
  rownames(colDat) <- colDat$sampleID
  
  mae <- lhCreateMAE(methylData,
      expresData, colData = colDat
  )
  
  sampleSize <- ncol(MultiAssayExperiment::experiments(mae)[[1]])
  reqPercentages <- matrix(c(3, 20, 5, 5, 40, 20, 4, 1, 2),
                           nrow = 3, byrow = TRUE
  )
  
  theWeightMifL <- matrix(c(2, -2, -sampleSize / 5, 1, 0, -2, 1, 1, 2),
                          nrow = 3, byrow = TRUE
  )
  
  theWeightMifNonL <- matrix(c(0, -2, -sampleSize / 5, 0, 0, -2, 0, 0, 0),
                             nrow = 3, byrow = TRUE
  )
  
  # Run the function
  scores <- scoreGenesMat(mae,
                          x1 = 1 / 3, x2 = 2 / 3,
                          y1 = NULL, y2 = NULL, percY1 = 1 / 3, percY2 = 2 / 3,
                          aReqPercentsMat = reqPercentages,
                          aWeightMifL = theWeightMifL,
                          aWeightMifNonL = theWeightMifNonL
  )
  
  # Check that the output is a data frame
  expect_s3_class(scores, "data.frame")
  
  # Check that it has the correct column names
  expect_true(all(c("logicSc", "numericSc") %in% colnames(scores)))
  
  # Check that logicSc is logical (TRUE/FALSE)
  expect_type(scores$logicSc, "logical")
  
  # Check that numericSc is numeric
  expect_type(scores$numericSc, "double")
  
  # Check that the number of rows matches the number of genes
  expect_equal(nrow(scores), nrow(methylData))
  
  # Ensure numeric scores are finite values
  expect_true(all(is.finite(scores$numericSc)))
})

## ----setup, include = FALSE---------------------------------------------------
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----loadLibs-----------------------------------------------------------------
library(Lheuristic)

# library(kableExtra)
# library(VennDiagram)

data("TCGAexpression")
data("TCGAmethylation")

## ----createMAE, warning=FALSE, message=FALSE----------------------------------
library(MultiAssayExperiment)

colDat <- DataFrame(sampleID = colnames(TCGAmethylation))
rownames(colDat) <- colDat$sampleID


# Construct MultiAssayExperiment
mae1 <- lhCreateMAE(xDat = TCGAmethylation, yDat = TCGAexpression,
    xName = "methylation", yName = "expression", colData = colDat)

# Display dataset names
print(names(experiments(mae1)))

## ----corMethod----------------------------------------------------------------
cl <- correlationSelection(mae1, type = "Spearman",
    pValCutoff = 0.25, rCutoff = -0.5, adj = TRUE)

correlationL <- cl[!is.na(cl$SigNegCorr) & 
    cl$SigNegCorr, ]
correlationNoL <- cl[!is.na(cl$SigNegCorr)& 
    !cl$SigNegCorr, ]

message(
    "The number of genes selected 
    with the correlation method is: ", sum(correlationL$SigNegCorr),
    "\n"
)

## ----corrDistri,fig.width = 6, fig.height=4-----------------------------------
d <- density(correlationL[, 1])
x2 <- data.frame(x = d$x, y = d$y)

library(ggplot2)
ggplot(x2, aes(x,y)) + geom_line() +
    labs(
    title = "Significant correlations in the TCGA dataset",
    x = "Correlation",
    y = "Density"
    )+
    theme_minimal()


## ----showCorrs----------------------------------------------------------------
head(correlationL)

## ----plotGenesFromCorr1, fig.height=6, fig.width=6----------------------------
genes2plot <- rownames(correlationL)[1:3]

plotGenesMat(mae1, geneNames = genes2plot,
    text4Title = correlationL[rownames(correlationL), ""],
    saveToPDF = FALSE
)

## ----setWeights---------------------------------------------------------------
sampleSize <- dim(mae1[[2]])[2]
numGenes <- dim(mae1[[2]])[1]

reqPercentages <- matrix(c(2, 20, 5, 5, 40, 20, 3, 3, 2), 
    nrow = 3, byrow = TRUE)
sum(reqPercentages)
(maxminCounts <- toReqMat(sampleSize, reqPercentages))

(theWeightMifL <- matrix(c(2, -2, -sampleSize / 5, 1, 0, -2, 1, 1, 2), 
    nrow = 3, byrow = TRUE))
(theWeightMifNonL <- matrix(c(0, -2, -sampleSize / 5, 0, 0, -2, 0, 0, 0),
    nrow = 3,
    byrow = TRUE
))

heur <- scoreGenesMat(mae1,
    aReqPercentsMat = reqPercentages, aWeightMifL = theWeightMifL,
    aWeightMifNonL = theWeightMifNonL
)

message("Number of scatterplots scored  : ", dim(heur)[1], "\n")
message("Number of L-shape scatterplots : ", sum(heur[, 1]), "\n")

heurL <- heur[heur$logicSc, ]
heurNoL <- heur[!heur$logicSc, ]

## ----showTrueLs---------------------------------------------------------------
knitr::kable(heurL)

## ----plotTrueLs, fig.height=6, fig.width=6------------------------------------

genes2plot2 <-   rownames(heurL)[1:4] #  

plotGenesMat(mae1, geneNames = genes2plot2,
    fileName = NULL, text4Title = heurL[genes2plot2, "numeriSc"],
    saveToPDF = FALSE
)


## ----checkCommon--------------------------------------------------------------
inCommonL <- intersect(rownames(correlationL), 
    rownames(heurL))
inCorrelationLOnly <- setdiff(rownames(correlationL),
    inCommonL)
inheurLLOnly <- setdiff(rownames(heurL), inCommonL)

## ----plotSelected, fig.height=6, fig.width=6----------------------------------
par(mfrow = c(2, 2))
myGene1 <- inCommonL[1]
titleT <- paste(myGene1, "(May be GRM)")
plotGeneSel(mae1,myGene1,
    titleText = titleT,
    x1 = 1/3, x2 = 2/3)

myGene2 <- inCommonL[2]
titleT <- paste(myGene2, "(May be GRM)")
plotGeneSel(mae1, myGene2,
    titleText = titleT,
    x1 = 1/3, x2 = 2/3)


myGene3 <- inCommonL[3]
titleT <- paste(myGene3, "(May be GRM)")
plotGeneSel(mae1,myGene3,
    titleText = titleT,
    x1 = 1/3, x2 = 2/3)

myGene5 <- inCommonL[5]
titleT <- paste(myGene5, "(May be GRM)")
plotGeneSel(mae1, myGene5,
    titleText = titleT,
    x1 = 1/3, x2 = 2/3)

## -----------------------------------------------------------------------------
sessionInfo()

#' Expression data matrix obtained from TCGA, TCGA-COAD dataset
#'
#' A mxn matrix containing normalized gene expressions from TCGA
#'
#' @docType data
#'
#' @usage data(TCGAexpression)
#'
#' @return data matrix with gene expression data
#'
#' @source \url{TCGA.org}
"TCGAexpression"

#' Methylation data matrix obtained from TCGA, TCGA-COAD dataset
#'
#' A mxn matrix containing normalized methylation values from TCGA
#'
#' @docType data
#'
#' @usage data(TCGAmethylation)
#'
#' @return data matrix with gene methylation data
#'
#' @source \url{TCGA.org}
"TCGAmethylation"

#' correlationSelection: A vector correlation function calculator
#'
#' \code{correlationSelection} Uses the function matCorrs; 
#' given two matrices X (m,n), Y (m,n) this function computes 
#' Pearson and Spearman correlation coefficients and their significance p-values
#' for every pair of row vectors.
#' @param mae a MultiAssayExperiment object containing the methylation
#' @param type specifies the correlation to choose between Spearman
#' and Pearson. Default is Spearman.
#' @param adj logical variable indicating if the p-value returned should be
#' adjusted or not. Default is TRUE, it returns an adjusted p-value.
#' @param pValCutoff the upper limit used for the  p-value. Default is 0.05.
#' @param rCutoff the upper limit used for the correlation coefficient. 
#' Default is 0, no cut off.
#' @param sortByCorrs logical; if TRUE, results are ordered by ascending 
#' p-value. Default set to FALSE.
#'
#' @return dataframe with the correlations selected
#'
#' @keywords Correlation Selection Calculator
#' @export correlationSelection
#' @import stats
#' @importFrom energy dcor
#' @examples
#'
#' # Methylation data
#' methylData <- matrix(runif(50), nrow = 10)
#' colnames(methylData) <- paste0("samp", 1:ncol(methylData))
#' rownames(methylData) <- paste0("gene", 1:nrow(methylData))
#' # Expression data
#' expresData <- matrix(rnorm(50), nrow = 10)
#' colnames(expresData) <- paste0("samp", 1:ncol(methylData))
#' rownames(expresData) <- paste0("gene", 1:nrow(methylData))
#' # ColData
#' colDat <- data.frame(
#'     sampleID = colnames(methylData),
#'     name = letters[1:ncol(methylData)]
#' )
#'
#' rownames(colDat) <- colDat$sampleID
#' mae <- MultiAssayExperiment::MultiAssayExperiment(
#'     experiments = list(
#'         methylation = methylData,
#'         expression = expresData
#'     ),
#'     colData = colDat
#' )
#' correlationSelection(mae, pValCutoff = 0.25, rCutoff = 0.1, 
#' type = "Spearman", sortByCorrs = TRUE)

correlationSelection <- function(mae, type = "Spearman", adj = TRUE,
    pValCutoff = 0.05,
    rCutoff = 0, sortByCorrs = FALSE) {
    # Extract the methylation and expression data from the MAE
    X <- assay(mae, "methylation")
    Y <- assay(mae, "expression")

    # Calculate the correlation matrix
    corsMat <- matAllCorrs(X, Y, sortByCorrs = sortByCorrs)

    # Filter based on correlation type
    if (type == "Spearman") {
        selected <- corsMat[, c("r (Sp)", "p (Sp)", 
        "adj.Spear.Pval", "distCor")]
        lShaped <- if (adj) {
        (!is.na(selected[, "r (Sp)"]) & selected[, "r (Sp)"] < rCutoff) &
        (!is.na(selected[, "adj.Spear.Pval"]) & 
        selected[, "adj.Spear.Pval"] < pValCutoff)
    } else {
        (!is.na(selected[, "r (Sp)"]) & 
        selected[, "r (Sp)"] < rCutoff) &
        (!is.na(selected[, "p (Sp)"]) & 
        selected[, "p (Sp)"] < pValCutoff)
    }
    } else {
    selected <- corsMat[, c("r (Pear)", "p (Pear)", 
        "adj.Pear.Pval", "distCor")]
    lShaped <- if (adj) {
        (!is.na(selected[, "r (Pear)"]) & 
        selected[, "r (Pear)"] < rCutoff) &
        (!is.na(selected[, "adj.Pear.Pval"]) & 
        selected[, "adj.Pear.Pval"] < pValCutoff)
    } else {
        (!is.na(selected[, "r (Pear)"]) & 
        selected[, "r (Pear)"] < rCutoff) &
        (!is.na(selected[, "p (Pear)"]) & 
        selected[, "p (Pear)"] < pValCutoff)
    }
    }

    # Combine results and label significant correlations
    selected2 <- data.frame(selected, lShaped)
    colnames(selected2) <- c(colnames(selected), "SigNegCorr")
    return(selected2)
}

vecCorrs <- function(x, y) {
    corrS <- rcorr(x, y, type = "spearman")
    corrP <- rcorr(x, y, type = "pearson")
    return(unlist(list(
        rhoS = corrS$r[1, 2],
        rhoP = corrP$r[1, 2], pvalS = corrS$P[1, 2],
        pvalP = corrP$P[1, 2]
    )))
}

distCorrs <- function(x, y) {
    distCorr <- energy::dcor(x, y, index = 1)
    return(unlist(list(distCorr)))
}

matDistCorr <- function(X, Y) {
    if ((nrow(X) != nrow(Y)) || (ncol(X) != ncol(Y))) {
        stop("matrices dimensions do not match")
    }
    distcorL <- matrix(NA, nrow = nrow(X), ncol = 1)
    colnames(distcorL) <- c("DistCor")
    for (i in seq_len(nrow(X))) {
        DistCorr <- distCorrs(X[i, ], Y[i, ])
        distcorL[i, ] <- DistCorr
    }
    rownames(distcorL) <- rownames(X)
    return(distcorL)
}

allCorrs <- function(x, y) {
    corrS <- rcorr(x, y, type = "spearman")
    corrP <- rcorr(x, y, type = "pearson")
    distCorr <- energy::dcor(x, y, index = 1)
    return(unlist(list(
        rhoS = corrS$r[1, 2],
        rhoP = corrP$r[1, 2],
        distCorr, pvalS = corrS$P[1, 2], pvalP = corrP$P[1, 2]
    )))
}

sort1 <- function(X, col, DEC = TRUE, ...) {
    return(X[sort.list(X[, col], 
    decreasing = DEC), ])
}

matAllCorrs <- function(X, Y, sortByCorrs = FALSE) {
    if ((nrow(X) != nrow(Y)) || (ncol(X) != ncol(Y))) {
        stop("matrices dimensions do not match")
    }
    corL <- matrix(NA, nrow = nrow(X), ncol = 5)
    colnames(corL) <- c("r (Sp)", "r (Pear)", 
        "distCor", "p (Sp)", "p (Pear)")
    for (i in seq_len(nrow(X))) {
        corrs <- allCorrs(X[i, ], Y[i, ])
        corL[i, ] <- corrs
    }
    corL <- cbind(corL, 
        adj.Spear.Pval = p.adjust(corL[, "p (Sp)"], "fdr"))
    corL <- cbind(corL, 
        adj.Pear.Pval = p.adjust(corL[, "p (Pear)"], "fdr"))
    rownames(corL) <- rownames(X)
    if (sortByCorrs) {
        corL <- sort1(corL, 4, DEC = FALSE)
    }
    return(corL)
}


zeros <- function(x) {
    which(x == 0)
}
discard <- function(x, where, howMany) {
    return(length(zeros(x[where])) > howMany)
}

discardA <- function(x, percentage) {
    maxZeros <- ceiling(length(x) * percentage)
    return(discard(x, seq_len(length(x)), maxZeros))
}


countNAs <- function(X) {
    sumaX <- apply(X, 1, sum)
    # sum(is.na(sumaX))
    xAmbNAs <- X[is.na(sumaX), ]
    NAs <- apply(xAmbNAs, 1, function(unVec) {
        sum(is.na(unVec))
    })
}


read2 <- function(expresFName, metFName, dataDirectory = ".", 
    sepChar = ";", decChar = ".") {
    expres <- utils::read.table(
        file = file.path(dataDirectory, expresFName), header = TRUE,
        sep = sepChar, dec = decChar, row.names = 1
    )
    mets <- utils::read.table(
        file = file.path(dataDirectory, metFName), header = TRUE,
        sep = sepChar, dec = decChar, row.names = 1
    )
    if (checkPairing(expres, mets)) {
        result <- list(expres, mets)
    } else {
        result <- NULL
    }
    return(result)
}

#' lhCreateMAE: Create a MultiAssayExperiment 
#' with Methylation and Expression Data
#'
#' This function constructs a MultiAssayExperiment 
#' object using methylation 
#' and expression data, automatically ensuring
#' consistency in sample names, 
#' feature names, and exposing dataset names.
#' It internally uses 
#' \code{checkPairing} to validate that the provided
#' datasets have matching 
#' row and column names.
#'
#' @param xDat A numeric matrix containing methylation data 
#' (features in rows, samples in columns).
#' @param yDat A numeric matrix containing expression data 
#' (features in rows, samples in columns).
#' @param xName A string specifying the name of the methylation dataset. 
#' Default: "methylation".
#' @param yName A string specifying the name of the expression dataset. 
#' Default: "expression".
#' @param colData (Optional) A DataFrame containing sample-level metadata. 
#' Default: NULL.
#'
#' @return A MultiAssayExperiment object containing the provided datasets 
#' with sample metadata.
#'
#' @keywords MultiAssayExperiment, Data Integration, Omics Data
#' @export lhCreateMAE
#' @importFrom MultiAssayExperiment MultiAssayExperiment
#' @importFrom MultiAssayExperiment experiments
#' @importFrom MultiAssayExperiment ExperimentList
#' @importFrom MultiAssayExperiment assay
#' @examples
#' 
#' library(MultiAssayExperiment)
#' 
#' # Create synthetic methylation and expression data
#' methylData <- matrix(runif(50), nrow = 10)
#' colnames(methylData) <- paste0("samp", 1:ncol(methylData))
#' rownames(methylData) <- paste0("gene", 1:nrow(methylData))
#'
#' expresData <- matrix(rnorm(50), nrow = 10)
#' colnames(expresData) <- colnames(methylData)
#' rownames(expresData) <- rownames(methylData)
#'
#' # Create sample metadata
#' colDat <- data.frame(sampleID = colnames(methylData),
#' name = letters[1:ncol(methylData)])
#' rownames(colDat) <- colDat$sampleID
#'
#' # Construct MultiAssayExperiment
#' mae <- lhCreateMAE(methylData, expresData, colData = colDat)
#'
#' # Display dataset names
#' names(experiments(mae))
#'
lhCreateMAE <- function(xDat, yDat, xName = "methylation", 
    yName = "expression", colData = NULL) {

    # Validate inputs
    if (!is.matrix(xDat) || !is.matrix(yDat)) {
        stop("xDat and yDat must be numeric matrices")
    }

    # Ensure row and column names match
    if (!identical(rownames(xDat), rownames(yDat)) || 
    !identical(colnames(xDat), colnames(yDat))) {
    stop("Row and column names must match
    between xDat and yDat")
    }

    # Create sample metadata if not provided
    if (is.null(colData)) {
    colData <- DataFrame(sampleID = colnames(xDat))
    rownames(colData) <- colData$sampleID
    }

    # Explicitly create an ExperimentList with names
    experiments <- ExperimentList(setNames(list(xDat, yDat),
    c(xName, yName)))

    # Create MultiAssayExperiment
    mae <- MultiAssayExperiment(
    experiments = experiments,
    colData = colData
    )

    # Print dataset names
    message("Datasets included in the MultiAssayExperiment: ",
        paste(names(experiments(mae)), collapse = ", "))

    return(mae)
}

library(MultiAssayExperiment)
library(testthat)

test_that("lhCreateMAE creates a valid MultiAssayExperiment", {
  
  # Create synthetic methylation and expression data
  methylData <- matrix(runif(50), nrow = 10)
  colnames(methylData) <- paste0("samp", 1:ncol(methylData))
  rownames(methylData) <- paste0("gene", 1:nrow(methylData))
  
  expresData <- matrix(rnorm(50), nrow = 10)
  colnames(expresData) <- colnames(methylData)
  rownames(expresData) <- rownames(methylData)
  
  # Create sample metadata
  colDat <- data.frame(sampleID = colnames(methylData),
                       name = letters[1:ncol(methylData)])
  rownames(colDat) <- colDat$sampleID
  
  # Call function
  mae <- lhCreateMAE(methylData, expresData, colData = colDat)
  
  # Check that the output is a MultiAssayExperiment object
  expect_s4_class(mae, "MultiAssayExperiment")
  
  # Check that dataset names match the expected ones
  expect_equal(names(experiments(mae)), c("methylation", "expression"))
  
  # Check if colData was correctly assigned
  expect_equal(rownames(colData(mae)), colDat$sampleID)
  
  # Check if experiments were correctly stored
  expect_equal(dim(experiments(mae)[["methylation"]]), dim(methylData))
  expect_equal(dim(experiments(mae)[["expression"]]), dim(expresData))
})

test_that("lhCreateMAE works when row/column names match", {
    methylData <- matrix(runif(50), nrow = 10)
    colnames(methylData) <- paste0("samp", 1:ncol(methylData))
    rownames(methylData) <- paste0("gene", 1:nrow(methylData))
    expresData <- matrix(rnorm(50), nrow = 10)
    colnames(expresData) <- colnames(methylData)  # Match column names
    rownames(expresData) <- rownames(methylData)

    # Suppress messages and ensure no error occurs
    expect_silent(suppressMessages(lhCreateMAE(methylData, expresData)))
    })

test_that("lhCreateMAE assigns default colData if missing", {
  methylData <- matrix(runif(50), nrow = 10)
  colnames(methylData) <- paste0("samp", 1:ncol(methylData))
  rownames(methylData) <- paste0("gene", 1:nrow(methylData))
  
  expresData <- matrix(rnorm(50), nrow = 10)
  colnames(expresData) <- colnames(methylData)
  rownames(expresData) <- rownames(methylData)
  
  mae <- lhCreateMAE(methylData, expresData)  # No colData provided
  
  expect_s4_class(mae, "MultiAssayExperiment")
  expect_equal(rownames(colData(mae)), colnames(methylData))
})

#' plotGenesMat
#'
#' \code{plotGenesMat} wrapper function for plotting the scatterplots associated
#' with two matrices.
#'
#' @param mae A MultiAssayExperiment object containing the methylation
#' and expression data.
#' @param geneNames A character vector of gene names to plot. 
#' If NULL, all genes are plotted.
#' @param fileName The name of the file used to save the results as a PDF.
#' If NULL, the plot is displayed on the screen.
#' @param text4Title An optional title for the plot, incorporating the gene name
#' and L-shape score. Defaults to NULL.
#' @param x1 The x-coordinate of vertical points on the X-axis. 
#' Expected to contain methylation values ranging between 0 and 1,
#' with default values set to 1/3 and 2/3.
#' @param y1 The y-coordinate of vertical points on the Y-axis.
#' If NULL, these are set to the percentiles of yVec 
#' defined by percY1 and percY2.
#' @param percY1 Values used as defaults for y1 when it is set to NULL.
#' @param x2 The x-coordinate of vertical points on the X-axis.
#' Expected to contain methylation values ranging between 0 and 1,
#' with default values set to 1/3 and 2/3.
#' @param y2 The y-coordinate of vertical points on the Y-axis.
#' If NULL, these are set to the percentiles of yVec 
#' defined by percY1 and percY2.
#' @param percY2 Values used as defaults for y2 when it is set to NULL.
#' @param plotGrid A logical value; defaults to TRUE, indicating 
#' whether to plot gridlines on the graph.
#' @param logicSc A numeric score representing the L-shape score. 
#' Defaults to NULL.
#' @param plotsPerPage A numeric value indicating the number of plots
#' to be printed in each page. Default set to 9.
#' @param saveToPDF Logical, if TRUE, saves the plots to a PDF file 
#' specified by fileName. If FALSE (default), plots are displayed 
#' interactively without saving.
#'
#' @return a pdf with scatterplots for all genes
#'
#' @keywords scatterplot gene plot matrix
#' @importFrom grDevices pdf
#' @importFrom grDevices dev.off
#' @importFrom MultiAssayExperiment assay
#' @import ggplot2
#' @import ggpubr
#' @export plotGenesMat
#' @examples
#' # Methylation data
#' methylData <- matrix(runif(50), nrow = 10)
#' colnames(methylData) <- paste0("samp", 1:ncol(methylData))
#' rownames(methylData) <- paste0("gene", 1:nrow(methylData))
#' # Expression data
#' expresData <- matrix(rnorm(50), nrow = 10)
#' colnames(expresData) <- paste0("samp", 1:ncol(methylData))
#' rownames(expresData) <- paste0("gene", 1:nrow(methylData))
#' # ColData
#' colDat <- data.frame(
#' sampleID = colnames(methylData),
#' name = letters[1:ncol(methylData)])
#'
#' rownames(colDat) <- colDat$sampleID
#' mae <- MultiAssayExperiment::MultiAssayExperiment(
#'     experiments = list(
#'         methylation = methylData,
#'         expression = expresData
#'     ),
#'     colData = colDat
#' ) 
#' selectedGenes <- c("gene1", "gene5")

#' plotGenesMat(mae, geneNames = selectedGenes,
#' saveToPDF = FALSE)
#'

plotGenesMat <- function(mae, geneNames = NULL,
    fileName = "Scatplot_Sel_Genes.pdf", 
    text4Title = NULL, x1 = 1/3,x2 = 2/3, y1 = NULL, y2 = NULL,
    percY1 = 1/3, percY2 = 2/3, plotGrid = TRUE,logicSc = NULL,
    saveToPDF = FALSE, plotsPerPage = 9) {

    if (!is.null(geneNames)) {
    mae <- mae[geneNames, , ]}

    if (!is.null(text4Title)) {
    text4Title <- paste(rownames(assay(mae,"expression")),
    text4Title, sep = ", ")} else {
    text4Title <- if (is.null(logicSc)) {
        rownames(assay(mae, "expression"))
    } else {paste(rownames(assay(mae, "expression")),
        "\nL-shaped = ", logicSc, sep = " ")}
    }

    plotList <- list()

    for (gene in seq_len(nrow(assay(mae,"expression")))) {
    plotList[[gene]] <- plotGeneSel(mae = mae,
        genePos = gene, titleText = text4Title[gene],
        x1 = x1,x2 = x2, percY1 = percY1, percY2 = percY2,
        plotGrid = plotGrid)
    }

    if (saveToPDF) {
        grDevices::pdf(fileName)
    
    for (i in seq(1, length(plotList),
        by = plotsPerPage)) {
        ggarrange(plotlist = plotList[i:min(i + plotsPerPage - 1,
        length(plotList))], ncol = 3, 
        nrow = ceiling(plotsPerPage / 3))}

    grDevices::dev.off()} else {
    return(ggarrange(plotlist = plotList, ncol = 3,
        nrow = ceiling(length(plotList) / 3)))}
}

#' plotGeneSel
#'
#' \code{plotGeneSel} plots points on a scatterplot with a 3x3 grid overimposed.
#'
#' @param mae A MultiAssayExperiment object containing the methylation
#' and expression data for the specified gene.
#' @param genePos The index of the gene to be plotted within the
#' MultiAssayExperiment object.
#' @param titleText plot title.
#' @param x1,x2 Coordinates of vertical points in the X axis. 
#' Because it is expected to contain methylation values that vary
#' between 0 and 1. The default values are 1/3 and 2/3.
#' @param y1,y2 Coordinates of vertical points in the Y axis. 
#' Leaving them as NULL assigns them the percentiles of yVec defined by `percY1`
#' and `percY2`.
#' @param percY1,percY2 Values used to act as default for `y1`and `y2`
#' when these are set to `NULL`
#' @param plotGrid logical. Defautl to TRUE will plot gridlines over the 
#' scatterplot.
#'
#' @return a pdf with scatterplots for selected genes
#'
#' @keywords plot gene selection
#' @importFrom graphics abline
#' @importFrom MultiAssayExperiment assay
#' @import ggplot2
#' @export plotGeneSel
#'
#' @examples
#' # Methylation data
#' methylData <- matrix(runif(50), nrow = 10)
#' colnames(methylData) <- paste0("samp", 1:ncol(methylData))
#' rownames(methylData) <- paste0("gene", 1:nrow(methylData))
#' # Expression data
#' expresData <- matrix(rnorm(50), nrow = 10)
#' colnames(expresData) <- paste0("samp", 1:ncol(methylData))
#' rownames(expresData) <- paste0("gene", 1:nrow(methylData))
#' # ColData
#' colDat <- data.frame(
#'     sampleID = colnames(methylData),
#'     name = letters[1:ncol(methylData)]
#' )
#'
#' rownames(colDat) <- colDat$sampleID
#' mae <- MultiAssayExperiment::MultiAssayExperiment(
#'     experiments = list(
#'         methylation = methylData,
#'         expression = expresData
#'     ),
#'     colData = colDat
#' )
#' 
#' plotGeneSel(mae, genePos = 7, titleText = "L-shaped gene")
#'
plotGeneSel <- function(mae, genePos, titleText, x1 = 1/3, 
    x2 = 2/3, y1 = NULL, y2 = NULL, percY1 = 1/3, 
    percY2 = 2/3, plotGrid = TRUE) {
    xVec <- as.numeric(MultiAssayExperiment::assay(mae, 
        "methylation")[genePos, ])
    yVec <- as.numeric(MultiAssayExperiment::assay(mae, 
        "expression")[genePos, ])
    minExp <- min(yVec)
    maxExp <- max(yVec)
    delta <- maxExp - minExp
    # Create a data frame for ggplot
    Methylation <- xVec
    Expression <- yVec
    plotData <- data.frame(Methylation, Expression)

    # Set y1 and y2 based on percentiles if not provided
    if (is.null(y1)) {
        y1 <- minExp + percY1 * delta
    }
    if (is.null(y2)) {
        y2 <- minExp + percY2 * delta
    }

    # Base ggplot scatterplot
    p <- ggplot2::ggplot(plotData, ggplot2::aes(x = Methylation, 
    y = Expression)) +
    ggplot2::geom_point() +
    ggplot2::labs(title = titleText) +
    ggplot2::xlim(0, 1) +
    ggplot2::ylim(minExp, maxExp)

    # Add grid lines if plotGrid is TRUE
    if (plotGrid) {
        p <- p +
        ggplot2::geom_vline(xintercept = c(x1, x2), 
        linetype = "dashed", color = "gray") +
        ggplot2::geom_hline(yintercept = c(y1, y2), 
        linetype = "dashed", color = "gray") + 
        ggplot2::theme_minimal()
    }

    # Show the plot
    p
}

---
title: "L-shaped selection from expression and methylation data"
author: "Berta Miro Cau and Alex Sánchez-Pla"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette: default
vignette: > 
  %\VignetteIndexEntry{L-shaped selection from methylation and expression}
  %\VignetteEncoding{UTF-8}  
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include = FALSE}
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

# Introduction
This vignette provides examples of how to use the `LHeuristic` package, 
designed to identify genes with an L-shaped pattern in scatterplots 
comparing gene expression and methylation.

The package enables users to:

- Data Input

    - Load gene expression and methylation values from CSV files
    
- Data Preprocessing

    - Set parameters for data usage, choosing either all genes or those meeting 
    specific filters (e.g., $\mbox{corr}< 0$)
    - Select the L-shape detection method and configure its parameters
    
- Data Processing

    - Run computations to identify L-shaped genes
    - Generate scatterplots for all genes or filtered selections
    
- Data Output

    - Save the list of identified genes and scatterplots


# Input data

The data is a subset of methylation and expression arrays from the 
colon adenocarcinoma (COAD) dataset in The Cancer Genome Atlas (TCGA). 
The full dataset is available on the TCGA website.

Methylation and expression data are in two separate files. 
Columns represent patient samples, and rows represent genes.

```{r loadLibs}
library(Lheuristic)

# library(kableExtra)
# library(VennDiagram)

data("TCGAexpression")
data("TCGAmethylation")
```

## Using `lhCreateMAE` based on `MultiAssayExperiment` as data container

To effectively integrate and manage omics data,
we utilize the `MultiAssayExperiment` data structure.
This approach ensures consistency between datasets and 
facilitates downstream analysis.

The `lhCreateMAE` function constructs a MultiAssayExperiment
object using two datasets—typically methylation and
expression data—while enforcing consistency in sample
and feature names. It ensures:

The datasets have matching row and column names.

Experiments are labeled appropriately
(e.g., "methylation" and "expression").

Sample metadata can be incorporated
via the colData argument.

The function includes an internal validation step,
which verifies that both datasets are aligned correctly.
If discrepancies exist in row or column names, the
function will return an error, prompting users to
correct inconsistencies before proceeding.


```{r createMAE, warning=FALSE, message=FALSE}
library(MultiAssayExperiment)

colDat <- DataFrame(sampleID = colnames(TCGAmethylation))
rownames(colDat) <- colDat$sampleID


# Construct MultiAssayExperiment
mae1 <- lhCreateMAE(xDat = TCGAmethylation, yDat = TCGAexpression,
    xName = "methylation", yName = "expression", colData = colDat)

# Display dataset names
print(names(experiments(mae1)))
```

# Data analysis

The `LHeuristic` package implements the heuristic algorithm by 
Sanchez et al. (2019) to detect L-shaped scatterplots. However, since many 
researchers rely on negative correlation to identify genes "regulated by 
methylation," the package includes both methods for completeness.

Each method has various parameters optimized for selecting L-shaped patterns 
and detecting negative correlations in scatterplot data.

The functions also support parameter tuning, allowing for other forms of 
correlation selection.

## Correlation method

The *correlation method* is a simple approach that identifies genes with a
significant negative correlation between their expression and methylation 
values.

```{r corMethod}
cl <- correlationSelection(mae1, type = "Spearman",
    pValCutoff = 0.25, rCutoff = -0.5, adj = TRUE)

correlationL <- cl[!is.na(cl$SigNegCorr) & 
    cl$SigNegCorr, ]
correlationNoL <- cl[!is.na(cl$SigNegCorr)& 
    !cl$SigNegCorr, ]

message(
    "The number of genes selected 
    with the correlation method is: ", sum(correlationL$SigNegCorr),
    "\n"
)
```

We can observe the distribution of the correlation coefficients:

```{r corrDistri,fig.width = 6, fig.height=4}
d <- density(correlationL[, 1])
x2 <- data.frame(x = d$x, y = d$y)

library(ggplot2)
ggplot(x2, aes(x,y)) + geom_line() +
    labs(
    title = "Significant correlations in the TCGA dataset",
    x = "Correlation",
    y = "Density"
    )+
    theme_minimal()

```

The number of genes retained depends on the *p-value* and *r* cutoff.

The result is a table listing "selected" and "unselected" genes, with columns
for the *r coefficient*, *p-value*, *adjusted p-value*, 
*distance correlation*, and a Boolean indicating whether the gene was selected
based on a significantly negative correlation.

```{r showCorrs}
head(correlationL)
```

The selected genes can be plotted and saved as a PDF (leaving the file name 
empty displays the plot on screen). The figure below shows the first four 
selected genes.

```{r plotGenesFromCorr1, fig.height=6, fig.width=6}
genes2plot <- rownames(correlationL)[1:3]

plotGenesMat(mae1, geneNames = genes2plot,
    text4Title = correlationL[rownames(correlationL), ""],
    saveToPDF = FALSE
)
```


To plot scatterplots showing the relationship between methylation and expression
for a list of genes, use the function `plotGenesMat`.

## Heuristic method

The heuristic method identifies L-shaped scatterplots by overlaying a grid 
on the graph and defining specific cells that must contain a minimum 
(or maximum) percentage of points for the scatterplot to qualify as L-shaped.

It computes a score where points in the designated L region receive 
positive values, while points outside this region receive negative values.
Properly adjusting scores and weights results in positive scores for 
L-shaped scatterplots and negative scores for non-L-shaped ones.

This concept can be clarified with a "three-band rule":

1. Overlay a \(3 \times 3\) grid on the scatterplot.

2. Classify the scatterplot as **"L" or "non-L"** based on a few conditions    
    2.1. A *minimum* number of points must be in the upper-left (cell (1,1))
    and lower-right (cell (3,3)) corners.
    2.2. A *maximum* number of points must be in the upper-right (cell (1,3)), 
    as points here indicate hypermethylation and hyperexpression, which are 
    contrary to our goal.
    2.3. A minimum of points in cell (3,1) is usually *not required* 
    unless we strictly want an L-shape; we can also accept diagonals 
    that reflect a negative correlation.

3. Score points in each sub-grid as follows:
    3.1. Points in permitted regions (left margin: cells (1,1), (2,2), (3,1),
    (3,2), (3,3)) score positively if classified as L, or zero 
    if classified as non-L.
    3.2. Points in non-desired regions (outer band: cells (1,2), (1,3), 
    (2,3)) always score negatively.
    3.3. Some regions, like cell (2,2), may be neutral and not score.

4. *Tune scoring parameters either manually (based on experience and dataset*
*characteristics) or through cross-validation* 
(**if a set of positive and negative L-shaped genes is available**).

The scheme can be summarized using the following equation:


$$
S(X) = W_L \circ X \times \mathbf{1}_L(X) + W_{L^C} \circ X \times 
\mathbf{1}_{L^c}(X)
$$ 
where:


- \(X\) is the matrix of *counts*, representing the number of counts in each 
cell of the grid.
- \(W_L\) is the matrix of scores per cell and point if the scatterplot is 
classified as \(L\).
- \(W_{L^c}\) is the matrix of scores per cell and point if the scatterplot is 
classified as non-\(L\) (\(L^c\)). 
The symbol \(\circ\) denotes the Hadamard product of the two
matrices \(W_{L/L^c}\) (i.e., element-wise multiplication), 
and \(\mathbf{1}_{L/L^c}()\) is
the indicator function for \(L\) or \(L^c\).

The classification of the scatterplot as \(L\) or \(L^c\) can also be 
represented by the Hadamard product of three matrices:

\[
\mathbf{1}_L(X) = \bigwedge_{i,j} X \circ C \circ 
\left( mMP \times \sum_{i,j} x_{ij}
\right),
\]

where:
- \(X\) is the matrix of *counts*, indicating the number of counts 
in each grid cell.

- \(C\) is the matrix of conditions to verify if the scatterplot 
should be classified as \(L\).
- \(mMP\) is the matrix of minimum and maximum percentages of points 
required in each cell for classification as \(L\).
- \(\circ\) represents point-wise logical operations, allowing the product
of the three matrices to act as a logical operation.
- \(\bigwedge_{i,j}\) denotes a logical "AND" operation across all cells;
if all cells are TRUE, the result is assigned to \(L\); if any cell fails,
it is assigned to \(L^c\).


```{r setWeights}
sampleSize <- dim(mae1[[2]])[2]
numGenes <- dim(mae1[[2]])[1]

reqPercentages <- matrix(c(2, 20, 5, 5, 40, 20, 3, 3, 2), 
    nrow = 3, byrow = TRUE)
sum(reqPercentages)
(maxminCounts <- toReqMat(sampleSize, reqPercentages))

(theWeightMifL <- matrix(c(2, -2, -sampleSize / 5, 1, 0, -2, 1, 1, 2), 
    nrow = 3, byrow = TRUE))
(theWeightMifNonL <- matrix(c(0, -2, -sampleSize / 5, 0, 0, -2, 0, 0, 0),
    nrow = 3,
    byrow = TRUE
))

heur <- scoreGenesMat(mae1,
    aReqPercentsMat = reqPercentages, aWeightMifL = theWeightMifL,
    aWeightMifNonL = theWeightMifNonL
)

message("Number of scatterplots scored  : ", dim(heur)[1], "\n")
message("Number of L-shape scatterplots : ", sum(heur[, 1]), "\n")

heurL <- heur[heur$logicSc, ]
heurNoL <- heur[!heur$logicSc, ]
```

We can check the results in the following table, which includes a logical value
indicating whether the gene exhibits an L-shape based on our criteria, 
along with the *numerSc* score.


```{r showTrueLs}
knitr::kable(heurL)
```

Next, we can visualize the scatterplots of the selected genes or save them
on a PDF file.

```{r plotTrueLs, fig.height=6, fig.width=6}

genes2plot2 <-   rownames(heurL)[1:4] #  

plotGenesMat(mae1, geneNames = genes2plot2,
    fileName = NULL, text4Title = heurL[genes2plot2, "numeriSc"],
    saveToPDF = FALSE
)

```


In summary, the Heuristic method allows us to select genes with an L-shaped 
scatterplot as follows:

1. **Select datasets:** A pair of row-column matched matrices for expression and
methylation.

2. **Set parameters:**
    2.1. Grid definition
    2.2. Binary scoring
    2.3. Numerical scoring

3. **Score the selected data:** Return scores, the classification 
(L-Shape = TRUE / non-L-shape = FALSE), and plots for each gene.

# Comparison of Selected Gene Lists

After identifying genes using the two methods, we can create the intersection 
of all lists and visualize the results with a Venn Diagram.

We may choose genes selected by two or more methods to ensure greater 
consistency in the selection.


```{r checkCommon}
inCommonL <- intersect(rownames(correlationL), 
    rownames(heurL))
inCorrelationLOnly <- setdiff(rownames(correlationL),
    inCommonL)
inheurLLOnly <- setdiff(rownames(heurL), inCommonL)
```


We can also plot selected genes.
As an example we will plot the genes 1, 2, 3 and 5 by index.

```{r plotSelected, fig.height=6, fig.width=6}
par(mfrow = c(2, 2))
myGene1 <- inCommonL[1]
titleT <- paste(myGene1, "(May be GRM)")
plotGeneSel(mae1,myGene1,
    titleText = titleT,
    x1 = 1/3, x2 = 2/3)

myGene2 <- inCommonL[2]
titleT <- paste(myGene2, "(May be GRM)")
plotGeneSel(mae1, myGene2,
    titleText = titleT,
    x1 = 1/3, x2 = 2/3)


myGene3 <- inCommonL[3]
titleT <- paste(myGene3, "(May be GRM)")
plotGeneSel(mae1,myGene3,
    titleText = titleT,
    x1 = 1/3, x2 = 2/3)

myGene5 <- inCommonL[5]
titleT <- paste(myGene5, "(May be GRM)")
plotGeneSel(mae1, myGene5,
    titleText = titleT,
    x1 = 1/3, x2 = 2/3)
```

# References

Sanchez-Pla A, Miró B, Carmona F et al. 
A heuristic algorithm to select genes potentially regulated by methylation
[version 1;  not peer reviewed]. 
F1000Research 2019, 8:1017.
(https://doi.org/10.7490/f1000research.1116986.1)

# Session info

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotGenesMat.R
\name{plotGenesMat}
\alias{plotGenesMat}
\title{plotGenesMat}
\usage{
plotGenesMat(
  mae,
  geneNames = NULL,
  fileName = "Scatplot_Sel_Genes.pdf",
  text4Title = NULL,
  x1 = 1/3,
  x2 = 2/3,
  y1 = NULL,
  y2 = NULL,
  percY1 = 1/3,
  percY2 = 2/3,
  plotGrid = TRUE,
  logicSc = NULL,
  saveToPDF = FALSE,
  plotsPerPage = 9
)
}
\arguments{
\item{mae}{A MultiAssayExperiment object containing the methylation
and expression data.}

\item{geneNames}{A character vector of gene names to plot. 
If NULL, all genes are plotted.}

\item{fileName}{The name of the file used to save the results as a PDF.
If NULL, the plot is displayed on the screen.}

\item{text4Title}{An optional title for the plot, incorporating the gene name
and L-shape score. Defaults to NULL.}

\item{x1}{The x-coordinate of vertical points on the X-axis. 
Expected to contain methylation values ranging between 0 and 1,
with default values set to 1/3 and 2/3.}

\item{x2}{The x-coordinate of vertical points on the X-axis.
Expected to contain methylation values ranging between 0 and 1,
with default values set to 1/3 and 2/3.}

\item{y1}{The y-coordinate of vertical points on the Y-axis.
If NULL, these are set to the percentiles of yVec 
defined by percY1 and percY2.}

\item{y2}{The y-coordinate of vertical points on the Y-axis.
If NULL, these are set to the percentiles of yVec 
defined by percY1 and percY2.}

\item{percY1}{Values used as defaults for y1 when it is set to NULL.}

\item{percY2}{Values used as defaults for y2 when it is set to NULL.}

\item{plotGrid}{A logical value; defaults to TRUE, indicating 
whether to plot gridlines on the graph.}

\item{logicSc}{A numeric score representing the L-shape score. 
Defaults to NULL.}

\item{saveToPDF}{Logical, if TRUE, saves the plots to a PDF file 
specified by fileName. If FALSE (default), plots are displayed 
interactively without saving.}

\item{plotsPerPage}{A numeric value indicating the number of plots
to be printed in each page. Default set to 9.}
}
\value{
a pdf with scatterplots for all genes
}
\description{
\code{plotGenesMat} wrapper function for plotting the scatterplots associated
with two matrices.
}
\examples{
# Methylation data
methylData <- matrix(runif(50), nrow = 10)
colnames(methylData) <- paste0("samp", 1:ncol(methylData))
rownames(methylData) <- paste0("gene", 1:nrow(methylData))
# Expression data
expresData <- matrix(rnorm(50), nrow = 10)
colnames(expresData) <- paste0("samp", 1:ncol(methylData))
rownames(expresData) <- paste0("gene", 1:nrow(methylData))
# ColData
colDat <- data.frame(
sampleID = colnames(methylData),
name = letters[1:ncol(methylData)])

rownames(colDat) <- colDat$sampleID
mae <- MultiAssayExperiment::MultiAssayExperiment(
    experiments = list(
        methylation = methylData,
        expression = expresData
    ),
    colData = colDat
) 
selectedGenes <- c("gene1", "gene5")
plotGenesMat(mae, geneNames = selectedGenes,
saveToPDF = FALSE)

}
\keyword{gene}
\keyword{matrix}
\keyword{plot}
\keyword{scatterplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotGeneSel.R
\name{plotGeneSel}
\alias{plotGeneSel}
\title{plotGeneSel}
\usage{
plotGeneSel(
  mae,
  genePos,
  titleText,
  x1 = 1/3,
  x2 = 2/3,
  y1 = NULL,
  y2 = NULL,
  percY1 = 1/3,
  percY2 = 2/3,
  plotGrid = TRUE
)
}
\arguments{
\item{mae}{A MultiAssayExperiment object containing the methylation
and expression data for the specified gene.}

\item{genePos}{The index of the gene to be plotted within the
MultiAssayExperiment object.}

\item{titleText}{plot title.}

\item{x1, x2}{Coordinates of vertical points in the X axis. 
Because it is expected to contain methylation values that vary
between 0 and 1. The default values are 1/3 and 2/3.}

\item{y1, y2}{Coordinates of vertical points in the Y axis. 
Leaving them as NULL assigns them the percentiles of yVec defined by `percY1`
and `percY2`.}

\item{percY1, percY2}{Values used to act as default for `y1`and `y2`
when these are set to `NULL`}

\item{plotGrid}{logical. Defautl to TRUE will plot gridlines over the 
scatterplot.}
}
\value{
a pdf with scatterplots for selected genes
}
\description{
\code{plotGeneSel} plots points on a scatterplot with a 3x3 grid overimposed.
}
\examples{
# Methylation data
methylData <- matrix(runif(50), nrow = 10)
colnames(methylData) <- paste0("samp", 1:ncol(methylData))
rownames(methylData) <- paste0("gene", 1:nrow(methylData))
# Expression data
expresData <- matrix(rnorm(50), nrow = 10)
colnames(expresData) <- paste0("samp", 1:ncol(methylData))
rownames(expresData) <- paste0("gene", 1:nrow(methylData))
# ColData
colDat <- data.frame(
    sampleID = colnames(methylData),
    name = letters[1:ncol(methylData)]
)

rownames(colDat) <- colDat$sampleID
mae <- MultiAssayExperiment::MultiAssayExperiment(
    experiments = list(
        methylation = methylData,
        expression = expresData
    ),
    colData = colDat
)

plotGeneSel(mae, genePos = 7, titleText = "L-shaped gene")

}
\keyword{gene}
\keyword{plot}
\keyword{selection}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/toReqMat.R
\name{toReqMat}
\alias{toReqMat}
\title{toReqMat}
\usage{
toReqMat(numPoints, aReqPercentMat)
}
\arguments{
\item{numPoints}{Number of points in a scatterplot. Used to turn the required
percentages into required counts.}

\item{aReqPercentMat}{Matrix of required percentages.}
}
\value{
a counts matrix
}
\description{
\code{toReqMat} can be used to turn a matrix of required percentages into
a matrix of required counts to facilitate its scoring.
}
\examples{
reqPercentages <- matrix(
    c(
        3, 20, 5,
        5, 40, 20,
        4, 1, 2
    ),
    nrow = 3, byrow = TRUE
)
numberOfPoints <- 100
reqMat <- toReqMat(
    numPoints = numberOfPoints,
    aReqPercentMat = reqPercentages
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binScore.R
\name{binScore}
\alias{binScore}
\title{binScore}
\usage{
binScore(aGrid, aReq)
}
\arguments{
\item{aGrid}{A matrix of counts as computed by `calcFreqs` function.}

\item{aReq}{A matrix of minimum or maximum counts to be found in each cell
if L-shape is TRUE.}
}
\value{
a score value for each scatterplot
}
\description{
\code{binScore} can be used to score scatterplots by directly comparing
the sample counts with a matrix of minimal or maximal percentages/counts
to be found in each cell. It implements the three bands rule implicitly
by setting threshold values.
}
\examples{
# Generate some example data
aGrid <- matrix(c(20, 3, 0, 10, 2, 2, 20, 10, 20),
    nrow = 3, ncol = 3, byrow = TRUE
)
aReq <- matrix(c(15, 5, 0, 0, 5, 5, 10, 10, 15),
    nrow = 3, ncol = 3, byrow = TRUE
)

# Calculate the bin score
binScore(aGrid, aReq)

}
\keyword{binary}
\keyword{scoring}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkPairing.R
\name{checkPairing}
\alias{checkPairing}
\title{checkPairing}
\usage{
checkPairing(X, Y)
}
\arguments{
\item{X}{matrix}

\item{Y}{matrix}
}
\value{
a logical value indicating if the two matrices have the 
same row and column names.
}
\description{
\code{checkPairing} function to check if two matrices have the same 
dimensions row and column names.
}
\examples{
(X <- round(matrix(rnorm(30) * 10, ncol = 6), 1)) + 1:10
(Y <- round(X + matrix(rnorm(30) * 10, ncol = 6), 1)) - 10:1
(rownames(X) <- rownames(Y) <- letters[1:nrow(X)])
(m1 <- checkPairing(X, Y))

}
\keyword{matrix}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcFreqs.R
\name{calcFreqs}
\alias{calcFreqs}
\title{calcFreqs}
\usage{
calcFreqs(
  mae,
  geneNum,
  x1,
  x2,
  y1 = NULL,
  y2 = NULL,
  percY1 = 1/3,
  percY2 = 2/3
)
}
\arguments{
\item{mae}{MultiAssayExperiment object containing methylation and expression
matrices.}

\item{geneNum}{row of expression/methylation matrix for which the frequencies
will be computed.}

\item{x1, x2}{Coordinates of vertical points in the X axis. Because it is
expected to contain.
methylation values that vary between 0 and 1, default values are 1/3 and 2/3.}

\item{y1, y2}{Coordinates of vertical points on the Y-axis. If set to NULL,
they will be automatically assigned as the percentiles of `y` defined 
by `percY1` and `percY2`.}

\item{percY1, percY2}{Default values for `y1` and `y2` when set to `NULL`}
}
\value{
a matrix with calculated frequencies
}
\description{
\code{calcFreqs} Given a MultiAssayExperiment with methylation and
expression data, the function overlays a grid on a YMet ~ Xmet scatterplot
and returns a 3x3 matrix with point counts per grid cell based on  the
vertical and horizontal lines.
}
\examples{

# Methylation data
methylData <- matrix(runif(50), nrow = 10)
colnames(methylData) <- paste0(
    "samp",
    1:ncol(methylData)
)
rownames(methylData) <- paste0(
    "gene",
    1:nrow(methylData)
)
# Expression data
expresData <- matrix(rnorm(50), nrow = 10)
colnames(expresData) <- paste0(
    "samp",
    1:ncol(methylData)
)
rownames(expresData) <- paste0(
    "gene",
    1:nrow(methylData)
)
# ColData
colDat <- data.frame(
    sampleID = colnames(methylData),
    name = letters[1:ncol(methylData)]
)

rownames(colDat) <- colDat$sampleID
mae <- MultiAssayExperiment::MultiAssayExperiment(
    experiments = list(
        methylation = methylData,
        expression = expresData
    ),
    colData = colDat
)
geneRow <- 1
x1 <- 1 / 3
x2 <- 2 / 3
y1 <- NULL
y2 <- NULL
percY1 <- 1 / 3
percY2 <- 2 / 3

calcFreqs(
    mae = mae, geneNum = geneRow,
    x1, x2, y1, y2, percY1, percY2
)
}
\keyword{calculation}
\keyword{frequencies}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/messageTitle.R
\name{messageTitle}
\alias{messageTitle}
\title{messageTitle}
\usage{
messageTitle(aMessage, underChar = "-")
}
\arguments{
\item{aMessage}{text of the message.}

\item{underChar}{character to use for underlining the message.}
}
\value{
a message to console
}
\description{
\code{messageTitle} A  wrapper function for sending messages to console.
}
\examples{
messageTitle("Hello world", "$")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{TCGAmethylation}
\alias{TCGAmethylation}
\title{Methylation data matrix obtained from TCGA, TCGA-COAD dataset}
\format{
An object of class \code{matrix} (inherits from \code{array}) with 1000 rows and 30 columns.
}
\source{
\url{TCGA.org}
}
\usage{
data(TCGAmethylation)
}
\value{
data matrix with gene methylation data
}
\description{
A mxn matrix containing normalized methylation values from TCGA
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scoreGenesMat.R
\name{scoreGenesMat}
\alias{scoreGenesMat}
\title{scoreGenesMat}
\usage{
scoreGenesMat(
  mae,
  x1 = 1/3,
  x2 = 2/3,
  y1 = NULL,
  y2 = NULL,
  percY1 = 1/3,
  percY2 = 2/3,
  aReqPercentsMat,
  aWeightMifL = 0.5,
  aWeightMifNonL = 0.25
)
}
\arguments{
\item{mae}{MultiAssayExperiment object containing methylation and expression 
matrices.}

\item{x1, x2}{Coordinates of vertical points on the X-axis. 
Expected to contain methylation values ranging between 0 and 1,
with default values set to 1/3 and 2/3.}

\item{y1, y2}{Coordinates of vertical points on the Y-axis. If set to NULL,
they default to the percentiles of yVec as defined by `percY1` and `percY2`.}

\item{percY1, percY2}{Values used to act as default for `y1`and `y2` when
these are set to `NULL`.}

\item{aReqPercentsMat}{A matrix specifying the minimum and maximum percentage
counts required in each cell.}

\item{aWeightMifL}{A matrix of weights applied to score counts when the
scatterplot is classified as "L".}

\item{aWeightMifNonL}{A matrix of weights applied to score counts when the
scatterplot is classified as "non-L".}
}
\value{
A data frame with two columns: 'logicSc' (logical score indicating if
the gene is 'active') and 'numericSc' (a numerical score).
}
\description{
\code{scoreGenesMat} scores scatterplots using a binary and a numeric
schemes row-wise.
}
\examples{
# Score genes based on example data

# Methylation data
methylData <- matrix(runif(50), nrow = 10)
colnames(methylData) <- paste0("samp", 1:ncol(methylData))
rownames(methylData) <- paste0("gene", 1:nrow(methylData))
# Expression data
expresData <- matrix(rnorm(50), nrow = 10)
colnames(expresData) <- paste0("samp", 1:ncol(methylData))
rownames(expresData) <- paste0("gene", 1:nrow(methylData))
# ColData
colDat <- data.frame(
    sampleID = colnames(methylData),
    name = letters[1:ncol(methylData)]
)

rownames(colDat) <- colDat$sampleID
mae <- MultiAssayExperiment::MultiAssayExperiment(
    experiments = list(
        methylation = methylData,
        expression = expresData
    ),
    colData = colDat
)
sampleSize <- ncol(MultiAssayExperiment::experiments(mae)[[1]])
reqPercentages <- matrix(c(3, 20, 5, 5, 40, 20, 4, 1, 2),
    nrow = 3, byrow = TRUE
)
(theWeightMifL <- matrix(c(2, -2, -sampleSize / 5, 1, 0, -2, 1, 1, 2),
    nrow = 3, byrow = TRUE
))
(theWeightMifNonL <- matrix(c(0, -2, -sampleSize / 5, 0, 0, -2, 0, 0, 0),
    nrow = 3, byrow = TRUE
))
scoreGenesMat(mae,
    x1 = 1 / 3, x2 = 2 / 3,
    y1 = NULL, y2 = NULL, percY1 = 1 / 3, percY2 = 2 / 3,
    aReqPercentsMat = reqPercentages,
    aWeightMifL = theWeightMifL,
    aWeightMifNonL = theWeightMifNonL
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/numScore.R
\name{numScore}
\alias{numScore}
\title{numScore}
\usage{
numScore(aGrid, LShaped, aWeightMifL, aWeightMifNonL)
}
\arguments{
\item{aGrid}{A matrix of counts computed by the `calcFreqs` function.}

\item{LShaped}{A boolean indicating whether the scatterplot can be
classified as "L-shaped".}

\item{aWeightMifL}{A matrix of weights applied to score counts if
the scatterplot is classified as "L".}

\item{aWeightMifNonL}{A matrix of weights applied to score counts if
the scatterplot is classified as "non-L".}
}
\value{
a numeric score for each scatterplot
}
\description{
\code{numScore} A function to score scatterplot using a weight matrix.
The scoring does not incorporate logical conditions such as 'if xij < C ...'
}
\examples{
# Methylation data
methylData <- matrix(runif(50), nrow = 10)
colnames(methylData) <- paste0("samp", 1:ncol(methylData))
rownames(methylData) <- paste0("gene", 1:nrow(methylData))
# Expression data
expresData <- matrix(rnorm(50), nrow = 10)
colnames(expresData) <- paste0("samp", 1:ncol(methylData))
rownames(expresData) <- paste0("gene", 1:nrow(methylData))
# ColData
colDat <- data.frame(
    sampleID = colnames(methylData),
    name = letters[1:ncol(methylData)]
)

rownames(colDat) <- colDat$sampleID
mae <- MultiAssayExperiment::MultiAssayExperiment(
    experiments = list(
        methylation = methylData,
        expression = expresData
    ),
    colData = colDat
)
trueFreq <- calcFreqs(mae, x1 = 1 / 3, x2 = 2 / 3)
LShaped <- FALSE
weightsIfL <- matrix(c(2, -1, -99, 1, 0, -1, 1, 1, 2),
    nrow = 3, byrow = TRUE
)
weightsIfNonL <- matrix(c(0, -1, -99, 0, 0, -1, 0, 0, 0),
    nrow = 3, byrow = TRUE
)
numScore(
    trueFreq, LShaped,
    weightsIfL, weightsIfNonL
)

}
\keyword{scatterplot}
\keyword{weights}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lhCreateMAE.R
\name{lhCreateMAE}
\alias{lhCreateMAE}
\title{lhCreateMAE: Create a MultiAssayExperiment 
with Methylation and Expression Data}
\usage{
lhCreateMAE(
  xDat,
  yDat,
  xName = "methylation",
  yName = "expression",
  colData = NULL
)
}
\arguments{
\item{xDat}{A numeric matrix containing methylation data 
(features in rows, samples in columns).}

\item{yDat}{A numeric matrix containing expression data 
(features in rows, samples in columns).}

\item{xName}{A string specifying the name of the methylation dataset. 
Default: "methylation".}

\item{yName}{A string specifying the name of the expression dataset. 
Default: "expression".}

\item{colData}{(Optional) A DataFrame containing sample-level metadata. 
Default: NULL.}
}
\value{
A MultiAssayExperiment object containing the provided datasets 
with sample metadata.
}
\description{
This function constructs a MultiAssayExperiment 
object using methylation 
and expression data, automatically ensuring
consistency in sample names, 
feature names, and exposing dataset names.
It internally uses 
\code{checkPairing} to validate that the provided
datasets have matching 
row and column names.
}
\examples{

library(MultiAssayExperiment)

# Create synthetic methylation and expression data
methylData <- matrix(runif(50), nrow = 10)
colnames(methylData) <- paste0("samp", 1:ncol(methylData))
rownames(methylData) <- paste0("gene", 1:nrow(methylData))

expresData <- matrix(rnorm(50), nrow = 10)
colnames(expresData) <- colnames(methylData)
rownames(expresData) <- rownames(methylData)

# Create sample metadata
colDat <- data.frame(sampleID = colnames(methylData),
name = letters[1:ncol(methylData)])
rownames(colDat) <- colDat$sampleID

# Construct MultiAssayExperiment
mae <- lhCreateMAE(methylData, expresData, colData = colDat)

# Display dataset names
names(experiments(mae))

}
\keyword{Data}
\keyword{Integration,}
\keyword{MultiAssayExperiment,}
\keyword{Omics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matCorrs.R
\name{matCorrs}
\alias{matCorrs}
\title{A row-wise correlation function calculator}
\usage{
matCorrs(mae)
}
\arguments{
\item{mae}{A MultiAssayExperiment object containing the methylation
and expression data.}
}
\value{
matrix with a correlation value for each gene
}
\description{
\code{matCorrs} Given a MultiAssayExperiment object, the function
computes Pearson and Spearman correlation coefficients
and their significance p-values for every pair of row vectors.
}
\examples{
# Methylation data
methylData <- matrix(runif(50), nrow = 10)
colnames(methylData) <- paste0("samp", 1:ncol(methylData))
rownames(methylData) <- paste0("gene", 1:nrow(methylData))
# Expression data
expresData <- matrix(rnorm(50), nrow = 10)
colnames(expresData) <- paste0("samp", 1:ncol(methylData))
rownames(expresData) <- paste0("gene", 1:nrow(methylData))
# ColData
colDat <- data.frame(
    sampleID = colnames(methylData),
    name = letters[1:ncol(methylData)]
)

rownames(colDat) <- colDat$sampleID
mae <- MultiAssayExperiment::MultiAssayExperiment(
    experiments = list(
        methylation = methylData,
        expression = expresData
    ),
    colData = colDat
)
matCorrs(mae)

}
\keyword{correlation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotGeneByName.R
\name{plotGeneByName}
\alias{plotGeneByName}
\title{plotGeneByName}
\usage{
plotGeneByName(
  geneName,
  mae,
  filename = NULL,
  text4Title = NULL,
  plotGrid = TRUE,
  figs = c(2, 2)
)
}
\arguments{
\item{geneName}{The name of the gene to be plotted.}

\item{mae}{A MultiAssayExperiment object containing the methylation
and expression data for the specified gene.}

\item{filename}{If provided, the name of the file to save the results
as a PDF; defaults to NULL.}

\item{text4Title}{A string used as the main title for the plot. Defaults to 
`geneName` if set to NULL.}

\item{plotGrid}{A boolean parameter indicating whether to pass
the grid option to the `plotGeneSel` function.}

\item{figs}{A two-component vector defining the 2-dimensional structure of
the plots to be generated.}
}
\value{
a pdf with scatterplots of selected by gene name
}
\description{
\code{plotGeneByName} plots points on a scatterplot with a 3x3 grid
superimposed. The name of a the gene is provided jointly with the
MultiAssayExperiment object and used to select the row to be plotted.
}
\examples{
# Plot gene by name based on example data
# Methylation data
methylData <- matrix(runif(50), nrow = 10)
colnames(methylData) <- paste0("samp", 1:ncol(methylData))
rownames(methylData) <- paste0("gene", 1:nrow(methylData))
# Expression data
expresData <- matrix(rnorm(50), nrow = 10)
colnames(expresData) <- paste0("samp", 1:ncol(methylData))
rownames(expresData) <- paste0("gene", 1:nrow(methylData))
# ColData
colDat <- data.frame(
    sampleID = colnames(methylData),
    name = letters[1:ncol(methylData)]
)

rownames(colDat) <- colDat$sampleID
mae <- MultiAssayExperiment::MultiAssayExperiment(
    experiments = list(
        methylation = methylData,
        expression = expresData
    ),
    colData = colDat
)
plotGeneByName(gene = "gene7", mae = mae)

}
\keyword{gene}
\keyword{name}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correlationSelection.R
\name{correlationSelection}
\alias{correlationSelection}
\title{correlationSelection: A vector correlation function calculator}
\usage{
correlationSelection(
  mae,
  type = "Spearman",
  adj = TRUE,
  pValCutoff = 0.05,
  rCutoff = 0,
  sortByCorrs = FALSE
)
}
\arguments{
\item{mae}{a MultiAssayExperiment object containing the methylation}

\item{type}{specifies the correlation to choose between Spearman
and Pearson. Default is Spearman.}

\item{adj}{logical variable indicating if the p-value returned should be
adjusted or not. Default is TRUE, it returns an adjusted p-value.}

\item{pValCutoff}{the upper limit used for the  p-value. Default is 0.05.}

\item{rCutoff}{the upper limit used for the correlation coefficient. 
Default is 0, no cut off.}

\item{sortByCorrs}{logical; if TRUE, results are ordered by ascending 
p-value. Default set to FALSE.}
}
\value{
dataframe with the correlations selected
}
\description{
\code{correlationSelection} Uses the function matCorrs; 
given two matrices X (m,n), Y (m,n) this function computes 
Pearson and Spearman correlation coefficients and their significance p-values
for every pair of row vectors.
}
\examples{

# Methylation data
methylData <- matrix(runif(50), nrow = 10)
colnames(methylData) <- paste0("samp", 1:ncol(methylData))
rownames(methylData) <- paste0("gene", 1:nrow(methylData))
# Expression data
expresData <- matrix(rnorm(50), nrow = 10)
colnames(expresData) <- paste0("samp", 1:ncol(methylData))
rownames(expresData) <- paste0("gene", 1:nrow(methylData))
# ColData
colDat <- data.frame(
    sampleID = colnames(methylData),
    name = letters[1:ncol(methylData)]
)

rownames(colDat) <- colDat$sampleID
mae <- MultiAssayExperiment::MultiAssayExperiment(
    experiments = list(
        methylation = methylData,
        expression = expresData
    ),
    colData = colDat
)
correlationSelection(mae, pValCutoff = 0.25, rCutoff = 0.1, 
type = "Spearman", sortByCorrs = TRUE)
}
\keyword{Calculator}
\keyword{Correlation}
\keyword{Selection}
